import { Component } from '@angular/core';
import { TesService } from './tes.service';
import { AuthorisationService} from './authorisation.service';
import { ComputesitesService} from './computesites.service';
import {BehaviorSubject} from 'rxjs/BehaviorSubject';
import { MatSnackBar } from '@angular/material';
import { Computesite, Health } from './computesite';
import {SettingsService } from './settings.service';
import {BrowserWindowService} from './browser-window.service';
import {OverlayContainer} from '@angular/cdk/overlay';
import {NotificationsService} from './notifications.service';
import {ActivatedRoute, Router, NavigationEnd } from '@angular/router';
import {filter} from 'rxjs/operators';



@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.css']
})
export class AppComponent {
  title = 'Strudelv2';
  private snackBarRef: any;
  private currentroute: string = '';
  public loggedin: number = 0;
  public loggedout: number = 0;

  constructor(private tesService: TesService,
              private authService: AuthorisationService,
              private computeSitesService: ComputesitesService,
              private settingsService: SettingsService,
              private browserWindowService: BrowserWindowService,
              public snackBar: MatSnackBar,
              private overlayContainer: OverlayContainer,
              private route: ActivatedRoute,
              private router: Router,
              private notifications: NotificationsService) {

   };

  ngOnInit() {
    const isdefined = filter(v  => v !== null && v !== undefined)
    this.notifications.getMsg().subscribe(msg => this.displayMessage(msg));
    this.settingsService.theme$.subscribe((v) => this.setTheme(v))
    this.authService.loggedInAuthZ.pipe(
      isdefined
    ).subscribe((v) => { console.log('update this.loggedin'); this.loggedin = (<[]>v).length ; console.log(this.loggedin)} );
    this.authService.loggedOutAuthZ.pipe(
      isdefined
    ).subscribe((v) => { console.log('update this.loggedout'); this.loggedout = (<[]>v).length ; console.log(this.loggedout)} );
  }

  toggleMenu() {
    if (this.settingsService.menuToggle$.value == true) {
      this.settingsService.menuToggle$.next(false);
      return
    }
    this.settingsService.menuToggle$.next(true);
  }


  private displayMessage(msg) {
   if (msg === null || msg == "") {
     if (this.snackBarRef != undefined) {
       this.snackBarRef.dismiss()
     }
   } else {
     this.snackBarRef = this.snackBar.open(msg,'Dismiss');
   }
 }
  setTheme(theme: string)  {
    console.log('in app component set theme');
    console.log(theme);
    if (theme == null) {
      console.log('theme is null');
      theme = 'strudel-light-theme';
    }
    let container = document.querySelector('app-root');
    let classList = container.classList;
    var c;
    for (c of classList.value.split(' ')) {
      if (c.indexOf('strudel-') == 0) {
        console.log('remove class'+c);
        classList.remove(c);
      }
    }
    classList.add(theme);
  }

}
