import { Injectable } from '@angular/core';
import { Computesite, Strudelapp } from './computesite';
import {BehaviorSubject, of, combineLatest} from 'rxjs';
import { HttpClientModule, HttpClient, HttpHeaders, HttpErrorResponse } from '@angular/common/http';
import { Observable } from 'rxjs';
import { catchError, map, tap, filter} from 'rxjs/operators';
import { Identity } from './identity';
import { TesService } from './tes.service';
import { AuthorisationService} from './authorisation.service';
import { NotificationsService } from './notifications.service';



@Injectable({
  providedIn: 'root',
})
export class ComputesitesService {
  public computesites: BehaviorSubject<Computesite[]>;
  public identities: BehaviorSubject<Identity[]>;
  public appidentities: BehaviorSubject<Identity[]>;
  public ftidentities: BehaviorSubject<Identity[]>;
  private statusMsg: BehaviorSubject<any>;

  constructor(private tesService: TesService, private http: HttpClient, private authorisationService: AuthorisationService, private notifications: NotificationsService) {
    this.computesites = new BehaviorSubject<Computesite[]>([]);
    this.identities = new BehaviorSubject<Identity[]>(null);
    this.appidentities = new BehaviorSubject<Identity[]>(null);
    this.ftidentities = new BehaviorSubject<Identity[]>(null);

    //this.computesites.subscribe(computesites => this.getStrudelApps(computesites))
    this.computesites.subscribe((cs) => this.authorisationService.updateAgentContents());
    combineLatest([this.authorisationService.agentContents,this.computesites.pipe(filter((v) => v !== []))])
      .subscribe(([ac,computesites]) => this.updateIdentities(ac, computesites));
    this.identities.subscribe((ids) => this.getStrudelAppsIds(ids))
    this.getComputeSites();
  }

  public setStatusMsg(statusMsg: BehaviorSubject<any>) {
    this.statusMsg = statusMsg;
  }


      /*getStrudelApps(computesites: Computesite[]) {
    for (let s of computesites) {
      let headers = new HttpHeaders();
      let options = { headers: headers, withCredentials: false};
      if (s.appCatalogUri !== null) {
        this.http.get<Strudelapp[]>(s.appCatalogUri,options)
                         .pipe(catchError(this.handleError('getStrudelApps')))
                         .subscribe(resp => this.updateStrudelApps(s,resp));
      }
    }

  }*/

  getStrudelAppsIdentity(identity: Identity): Observable<Strudelapp[]> {
    let uri$ = this.getStrudelAppsUri(identity.site.appCatalogUri);
    let cmd$ = this.getStrudelAppsCmd(identity);
    return combineLatest([uri$, cmd$]).pipe(map(([uri,cmd]) => (uri.concat(cmd))))

  }

  getStrudelAppsIds(ids: Identity[]) {
    if (ids != undefined && ids != null) {
      for (let id of ids) {
        console.log('getting apps for',id);
        this.getStrudelAppsIdentity(id)
                           .subscribe(resp => this.updateStrudelApps(id.site,resp));
      }
    }
  }


  private getStrudelAppsUri(uri): Observable<Strudelapp[]> {
      if (uri !== null) {
        let headers = new HttpHeaders();
        let options = { headers: headers, withCredentials: false};
        return this.http.get<Strudelapp[]>(uri,options)
          .pipe(catchError(this.handleError('getStrudelApps', <Strudelapp[]>[])))
      }
      return of([] as Strudelapp[] )
  }

  private getStrudelAppsCmd(identity: Identity): Observable<Strudelapp[]> {
    if (identity.site.appCatalogCmd != null) {
      return (this.tesService.runCommand(identity,identity.site.appCatalogCmd) as Observable<Strudelapp[]>)
                    .pipe(catchError(this.handleError('getStrudelApps', <Strudelapp[]>[])))
    }
    return of([] as Strudelapp[])
  }


  storeLocalStrudelApps(name: string, apps: any) {
      localStorage.setItem(name+'-apps',JSON.stringify(apps))
  }

  removeLocalStrudelApps() {
      var keys = [];
      for (var key in localStorage) {
        var idx = key.indexOf('-apps');
        if (idx !== -1) {
            keys.push(key)
        }
      }
      for (var key in keys) {
          localStorage.removeItem(key);
      }
  }


  updateStrudelApps(site: Computesite, apps) {
    var sapps: Strudelapp[];
    var localapps: Strudelapp[];
    sapps = <Strudelapp[]>apps;

    localapps = JSON.parse(localStorage.getItem(site.name+'-apps'))
    if (localapps !== null)  {
      for (let a of localapps) {
        sapps.push(a);
      }
    }
    site.appCatalog.next(sapps);
  }

  private handleError<T> (operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error);
      return of(result as T);
    };
  }


  getComputeSites() {
    let headers = new HttpHeaders();
    let options = { headers: headers, withCredentials: false};
    this.http.get<Strudelapp[]>('./assets/config/computesites.json',options)
                     .pipe(catchError(this.handleError('getComputeSites')))
                     .subscribe(resp => this.updateComputeSites(resp));
  }

  storeLocalComputeSites(computesites: any) {
      try {
        localStorage.setItem('localcomputesites',JSON.stringify(computesites));
      } catch {
      }
      this.getComputeSites();
  }

  removeLocalComputeSites() {
      localStorage.removeItem('localcomputesites');
      this.getComputeSites();
  }

  updateComputeSites(resp) {
    var localcomputesites: Computesite[] = [];
    var computesites: Computesite[] = []
    try {
        localcomputesites = JSON.parse(localStorage.getItem('localcomputesites'))
    } catch {
        localcomputesites = []
    }
    if (localcomputesites !== null ) {
      for (let cs of localcomputesites) {
        cs.appCatalog  = new BehaviorSubject<Strudelapp[]>([]);
        computesites.push(cs);
      }
    }
    for (let cs of resp) {
      let computesite = <Computesite>cs;
      computesite.appCatalog = new BehaviorSubject<Strudelapp[]>([])
      computesites.push(computesite);
    }
    this.computesites.next(computesites);
  }

  returnComputeSites(): Computesite[] {
    return this.computesites.value;
  }

  private siteMatch(cert: any, cs: Computesite): string[] {
    var fp: string = (<Computesite>cs).cafingerprint;
    if ('Signing CA' in cert) {
      for (let ca of cert['Signing CA']) {
        if (ca.indexOf(fp) != -1) {
          return cert['Principals']
        }
      }
    }
    return [];
  }

  private updateIdentities(agentContents,computesites) {
    let certs = agentContents;
    var cs: Computesite;
    var identities: Identity[] = [];
    var appidentities: Identity[] = [];
    var ftidentities: Identity[] = [];
    // If the agent contents is set to null we are probably still updating it
    console.log('updateIdentities ... check your subscriptions');

    if (agentContents == null) {
      return
    }
    for (cs of computesites) {
      for (let i in certs) {
        let principals = this.siteMatch(certs[i],cs);
        for (let principal of principals) {
          if (principal != null) {
              let id = new Identity(principal,cs);
              identities.push(id);
              if (cs.appCatalogUri != null || cs.appCatalogCmd != null) {
                appidentities.push(id);
              }
              if (cs.dtn != null ) {
                ftidentities.push(id);
              }
            }
          }
      }
      }
      this.identities.next(identities);
      this.ftidentities.next(ftidentities);
      this.appidentities.next(appidentities);
    }

}
